#!/bin/bash

###########################################################
# General 
###########################################################
#
# SCRIPT NAME get-sites-list.sh
# SCRIPT VERSION 1.0
# 
# CREATION DATE 15-5-2016
# AUTHOR ohadm
#
# Description:
#
# This script gets a list of web sites by given properties based on vhost files this server serves. 
# 
#
###########################################################

# set the location of the vhost files here
VHOSTS_DIR="/opt/v"

if ! [ -d "$VHOSTS_DIR" ]; then
	echo "ERROR! '$VHOSTS_DIR' is not a valid dir."
	echo "Aborting ..."
	exit 1
fi

printAnArrLineByLine()
{
	# print output array
	
	OUTPUT=($@)
	COUNTER=1

	for i in "${OUTPUT[@]}"; do
		echo $i
	done
}

printAnArrWithSpaces()
{
	# print output array
	
	OUTPUT=($@)
	COUNTER=1

	for i in "${OUTPUT[@]}"; do
		printf "$i "

		EVEN=`expr $COUNTER % 2`		

		if [ "$EVEN" == 0 ]; then
			printf "\n"
		fi

		COUNTER=$(expr $COUNTER + 1)
	done

}

printUsageInfoAndExit()
{
	echo -e "ERROR! Wrong input.\n"
	echo -e "Usage: $0 <site_protocol> <data_type>\n"
	echo -e "'site_protocol' options: \n"
	echo -e "1) all (http and https)"
	echo -e "2) https_only (if a site is also available via http it won't be listed)"
	echo -e "3) https (includes sites that support both protocols)"
	echo -e "4) http (only http sites)"

	echo -e "\n'data_type' options: \n"
	echo -e "1) dirs (site directories)"
	echo -e "2) urls (site urls)"
	echo -e "3) main-urls (only main urls)"
	echo -e "4) all (urls and dirs - not always applicable)"

	echo -e "\nExamples:\n"
	
	echo -e "Example 1: $0 all urls (e.g. for 'curl' testing)"
	echo -e "Example 2: $0 https_only dirs (e.g. for updating secure cookie header)"
	echo -e "Example 3: $0 https urls (e.g. for checking ssl settings with 'openssl')"
	echo -e "Example 4: $0 https main-urls (e.g. for checking certificates status with 'openssl')"
	echo -e "Example 5: $0 all all (e.g. for getting a table of sites info)"
	
	exit 1
}

printASiteDirectoryTakenFromAVhostFile()
{
	if [ -s "$1" ]; then
		if ! [ -d "$1" ]; then
			SITE_DIR=`cat $1 | grep -i "directory" | grep -v "<\/" | awk '{print \$2}' | tr -d ">" | tr -d "\""`

			SITE_DIR=${SITE_DIR%/}

			echo $SITE_DIR
		fi
	fi
}

printSiteUrlsTakenFromAVhostFile()
{
	if [ -s "$1" ]; then
		if ! [ -d "$1" ]; then
			URLS_ARR=()
			
			echo $1 | grep "ssl" > /dev/null			
	
			if [ "$?" == "0" ]; then
				PREFIX="https://"
			else
				PREFIX="http://"
			fi

			MAIN_URL=`cat $1 | grep -i "ServerName" | awk '{print \$2}'`
			URLS_ARR+=($PREFIX$MAIN_URL)

			ALIASES=`cat $1 | grep -i "ServerAlias" | grep -v \# | awk '{\$1=""; print \$0}'`
			
			TOKENS_ARR=(${ALIASES//\\n/ })

			for i in "${TOKENS_ARR[@]}"; do
				URLS_ARR+=($PREFIX$i)
			done
		fi
	fi

	printAnArrLineByLine ${URLS_ARR[@]}
}

printATableOfUrlsAndDirsTakenFromAVhostFile()
{
	if [ -s "$1" ]; then
		if ! [ -d "$1" ]; then
			TABLE_ARR=()
	
			SITE_DIR=`printASiteDirectoryTakenFromAVhostFile $1`

			SITE_URLS_ARR=`printSiteUrlsTakenFromAVhostFile $1`
		
			# Bug Fix: the array gotten from the previous func
			# comes back as one line instead of an actual array
			# because adding '\n' to a variable doesn't work
			# hence re-tokenizing by space is needed here:
	
			TOKENS_ARR=(${SITE_URLS_ARR//\ / })

			for i in "${TOKENS_ARR[@]}"; do
				LINE=$i\ $SITE_DIR
				
				TABLE_ARR+=($i\ $SITE_DIR)
			done
		fi
	fi

	printAnArrWithSpaces ${TABLE_ARR[@]}
}

if [ "$1" == "" -o "$2" == "" ]; then
	printUsageInfoAndExit
else
	case $2 in
		"dirs") 
			CMD="grep -i -v redirect | xargs -I {} cat {} | grep -i directory | grep -v \"<\/\" | awk '{print \$2}' | awk -F\> '{print \$1}'"

			PREFIX="no"
		;;
		"urls") 
			CMD="grep -i -v redirect | xargs -I {} cat {} | grep -v \# | grep -i \"ServerName\|ServerAlias\" | awk '{print \$2}'"

			PREFIX="yes"
		;;
		"main-urls") 
			CMD="grep -i -v redirect | xargs -I {} cat {} | grep -v \# | grep -i \"ServerName\" | awk '{print \$2}'"

			PREFIX="yes"
		;;
		"all")
			if ! [ "$1" == "all" ]; then
				printUsageInfoAndExit  
			else
				TABLE="yes"
			fi
		;;
		*) printUsageInfoAndExit  
	   	;;
 	esac

	COUNTER=0

	case "$1" in

		"all"|"https_only")
			if [ "$TABLE" == "yes" ]; then
				for i in $(ls $VHOSTS_DIR/*.conf | grep -iv redirect); do
					printATableOfUrlsAndDirsTakenFromAVhostFile $i
				done
			else
				LIST_OF_ITEMS=`echo "ls $VHOSTS_DIR/*.conf | grep -v "ssl" | $CMD" | sh 2>/dev/null`
				if ! [ "$LIST_OF_ITEMS" == "" ]; then 
					for i in $(echo "$LIST_OF_ITEMS"); do
					
						if [ "$1" == "https_only" ]; then
							DELETE_ARR[$COUNTER]=$i
						else	
							if [ "$PREFIX" == "yes" ]; then
								SITES[$COUNTER]=http://$i
							else
								# trim the char '"' if it exists
								CURRENT_SITE=`echo $i | tr -d "\""`							
							
								# trim last slash if it exists
								CURRENT_SITE=${CURRENT_SITE%/}							
						
								SITES[$COUNTER]=$CURRENT_SITE
							fi
						fi
		
						COUNTER=$(expr $COUNTER + 1)
					done
				fi
			fi
			
			# the '&' tells bash not to exit the case and continue running the statements in it.
			;&	
		"https")
			
			# if the request is for 'all' sites then a prefix is needed but if the request is for https sites only then for 'openssl' to work properly we can't use a prefix so this condition removes it

			if [ "$1" == "https" ]; then
				PREFIX="no"
			fi

			LIST_OF_ITEMS=`echo "ls $VHOSTS_DIR/ssl*.conf | $CMD" | sh 2>/dev/null`

			if ! [ "$LIST_OF_ITEMS" == "" ]; then 
				for i in $(echo "$LIST_OF_ITEMS"); do
			
					if [ "$PREFIX" == "yes" ]; then
						SITES[$COUNTER]=https://$i
					else
						# trim the char '"' if it exists
						CURRENT_SITE=`echo $i | tr -d "\""`							
							
						# trim last slash if it exists
						CURRENT_SITE=${CURRENT_SITE%/}							
						
						SITES[$COUNTER]=$CURRENT_SITE
					fi

					COUNTER=$(expr $COUNTER + 1)
				done
			fi
			
			# remove duplicate items
			NO_DUPS_ARR=(`echo ${SITES[@]} | tr " " "\n" | sort | uniq`)

			if [ "$1" == "https_only" ]; then
				for i in "${DELETE_ARR[@]}"; do
					OUTPUT_ARR=(${NO_DUPS_ARR[@]/$i})
				done
			else
				OUTPUT_ARR=${NO_DUPS_ARR[@]}
			fi
		;;		
		*) printUsageInfoAndExit
		;;
	esac

	printAnArrLineByLine ${OUTPUT_ARR[@]}
fi
